@@ -2,11 +2,9 @@ import numpy as np
 
 class Loss:
     def calculate(self, output, y):
-        sample_losses = self.forward(self, output, y)
+        sample_losses = self.forward(output, y)
         data_loss = np.mean(sample_losses)
         return data_loss
-    def forward(self, y_pred, y_true):
-        pass
 
 class CategoricalCrossentropy(Loss):
     def forward(self, y_pred, y_true):
@@ -16,7 +14,7 @@ class CategoricalCrossentropy(Loss):
         if len(y_true.shape) == 1:
             correct_confidences = y_pred_clipped[range(samples), y_true]
 
-        elif len(y_true.shape) == 2: ## Incase user passes one-hot encoded vector.
+        elif len(y_true.shape) == 2: # Incase user passes one-hot encoded vector.
             correct_confidences = np.sum(y_pred_clipped*y_true, axis=1)
 
         negative_log_likelihoods = -np.log(correct_confidences)
@@ -28,4 +26,4 @@ class CategoricalCrossentropy(Loss):
         if len(y_true.shape) == 1:
             y_true = np.eye(labels)[y_true]
         self.dinputs = -y_true / dvalues
-        self.dinputs = self.dinputs / samples
\ No newline at end of file
+        self.dinputs = self.dinputs / samples
