@@ -1,23 +1,27 @@
 import numpy as np
-from .layers import Layer
+from . import layers
 from . import losses
+from . import optimizers
 import os
+import copy
 
 class Sequential:
-    def __init__(self, layers: list, loss: losses.Loss = losses.CategoricalCrossentropy) -> None:
+    def __init__(self, layers: list = [], loss: losses.Loss = losses.CategoricalCrossentropy) -> None:
         self.layers = layers
         self.loss_function = loss
         self.__loss = 0
         self.__least_loss = 99999
         self.__output = []
-        self.__best_weights = []
-        self.__best_biases = []
 
     @property
     def model_loss(self):
         return self.__least_loss
 
-    def fit(self, X, y, epoch: int, smooth_output: bool = False,  verbose: bool = False, iteration: int = 5000) -> list:
+    def add(self, layer: layers.Layer):
+        self.layers.append(layer)
+        return len(self.layers)
+
+    def fit(self, X, y, epoch: int, optimizer = optimizers.Adam(learning_rate=0.05, decay=5e-7), smooth_output: bool = False,  verbose: bool = False, iteration: int = 10000) -> list:
         """Function for training neural network
 
         Args:
@@ -32,69 +36,89 @@ class Sequential:
             list(float): The list of least loss at epoch
         """
         self.iteration = iteration
-        self.__best_weights = [layer.weights for layer in self.layers]
-        self.__best_biases = [layer.biases for layer in self.layers]
-        history = []
+        self.history = []
         whole_string = ""
+        self.__best_layers = []
         for epoch_number in range(epoch):
             my_str = ""
+            optimizer = optimizer
             for i in range(self.iteration):
-                self.__current_weights = []
-                self.__current_biases  = []
-                weights_list = []
-                bias_list = []
+                self.__current_layers = []
+                __layers = []
                 for count, layer in enumerate(self.layers):
                     if count == 0:
                         layer.forward(X)
                         activation = layer.activation
                         self.__output = activation.forward(activation, layer.output)
+                    elif (count == (len(self.layers)-1)): # last layer
+                        layer.forward(self.__output)
+                        loss_activation = layer.activation
+                        self.__loss = loss_activation.forward(layer.output, y)
+                        predictions = np.argmax(loss_activation.output, axis=1)
+                        if len(y.shape) == 2:
+                            y = np.argmax(y, axis=1)
+                        accuracy = np.mean(predictions==y)
+                        self.__accuracy = accuracy
+                        self.__output_after_forward_pass = loss_activation.output
                     else:
-                        layer.weights = layer.weights + 0.05 * np.random.randn(layer.shape[0], layer.shape[1])
-                        layer.biases = layer.biases + 0.05 * np.random.randn(1, layer.shape[1])
                         layer.forward(self.__output)
                         activation = layer.activation
                         self.__output = activation.forward(activation, layer.output)
-                    weights_list.append(layer.weights)
-                    bias_list.append(layer.biases)
-                self.__current_weights.append(weights_list)
-                self.__current_biases.append(bias_list)
-                loss_function = self.loss_function
-                self.__loss = loss_function.calculate(loss_function, self.__output, y)
-                predictions = np.argmax(self.__output, axis=1)
-                accuracy = np.mean(predictions==y)
-                self.__accuracy = accuracy
+                        self.__output = activation.output
+                    __layers.append(copy.deepcopy(layer))
+                self.__current_layers.append(__layers)
+
+                # if not i % 100:
+                #     print(f'epoch: {i}, ' +
+                #         f'acc: {self.__accuracy:.3f}, ' +
+                #         f'loss: {self.__loss:.3f}, ' +
+                #         f'lr: {optimizer.current_learning_rate}')
+
+                for count, layer in enumerate(list(reversed(self.layers))):
+                    if count == 0:
+                        loss_activation = layer.activation
+                        loss_activation.backward(self.__output_after_forward_pass, y)
+                        self.__dinputs = loss_activation.dinputs
+                        layer.backward(self.__dinputs)
+                        self.first__dinputs = layer.dinputs
+                    else:
+                        activation = layer.activation
+                        activation.backward(activation, self.first__dinputs)
+                        layer.backward(activation.dinputs)
+                        self.__dinputs = layer.dinputs
+
+                _layers = optimizer.optimize(self.layers)
+                del self.layers
+                self.layers = _layers
+                
                 if self.__loss < self.__least_loss:
-                    if verbose:
-                        perc_done = i / self.iteration
-                        eq_count = int(perc_done * 50)
-                        eq_left = 49 - eq_count
-                        my_str = f"Epoch {epoch_number+1}/{epoch}" + "[" + "="*eq_count + ">" + " "*eq_left + "] " + f"loss = {self.__loss :.4f}, {accuracy = :.4f}"
-                        os.system('cls')
-                        whole_string += my_str
-                        print(whole_string)
-                        whole_string = whole_string[0:-len(my_str)]
-                    self.__best_weights = self.__current_weights.copy()
-                    self.__best_biases = self.__current_biases.copy()
                     self.__least_loss = self.__loss
-                else:
-                    if (smooth_output and verbose):
+                    self.__best_layers = copy.deepcopy(self.__current_layers)
+
+                if verbose:
+                    if smooth_output:
                         perc_done = i / self.iteration
                         eq_count = int(perc_done * 50)
                         eq_left = 49 - eq_count
-                        my_str = f"Epoch {epoch_number+1}/{epoch}" + "[" + "="*eq_count + ">" + " "*eq_left + "] " + f"loss = {self.__least_loss :.4f}, {accuracy = :.4f}"
+                        my_str = f"Epoch {epoch_number+1}/{epoch}" + "[" + "="*eq_count + ">" + " "*eq_left + "] " + f"loss = {self.__loss :.4f}, {self.__accuracy = :.4f}"
                         os.system('cls')
                         whole_string += my_str
                         print(whole_string)
                         whole_string = whole_string[0:-len(my_str)]
-                    for count, layer in enumerate(self.layers):
-                        layer.weights = self.__best_weights[0][count]
-                        layer.biases = self.__best_biases[0][count]
-            whole_string = whole_string + f"Epoch {epoch_number+1}/{epoch}" + "[" + "="*50 + "] " + f"loss = {self.__least_loss :.4f}, accuracy = {self.__accuracy :.4f}" + "\n"
-            history.append(self.__least_loss)
+                    else:
+                        if not i % 100:
+                            perc_done = i / self.iteration
+                            eq_count = int(perc_done * 50)
+                            eq_left = 49 - eq_count
+                            my_str = f"Epoch {epoch_number+1}/{epoch}" + "[" + "="*eq_count + ">" + " "*eq_left + "] " + f"loss = {self.__loss :.4f}, {self.__accuracy = :.4f}"
+                            os.system('cls')
+                            whole_string += my_str
+                            print(whole_string)
+                            whole_string = whole_string[0:-len(my_str)]
+
+            self.history.append(self.__loss)
             print(f"Epoch {epoch_number+1}: loss = {self.__least_loss}")
-        os.system('cls')
-        print(whole_string)
-        return history
+        return self.history
 
     def predict(self, X):
         """Function for predicions.
@@ -107,17 +131,22 @@ class Sequential:
         """
         predictions = []
         for x_point in X:
-            for count, layer in enumerate(self.layers):
+            for count, layer in enumerate(self.__best_layers[0]):
                 if count == 0:
                     layer.forward(x_point)
                     activation = layer.activation
                     self.__output = activation.forward(activation, layer.output)
+                elif (count == (len(self.layers)-1)): # last layer
+                    layer.forward(self.__output)
+                    loss_activation = layer.activation
+                    self.__loss = loss_activation.activation.forward(layer.output)
+                    prediction = np.argmax(self.__output, axis=1)
                 else:
                     layer.forward(self.__output)
                     activation = layer.activation
                     self.__output = activation.forward(activation, layer.output)
-            predictions.append(self.__output)
-        return predictions
+            predictions.append(prediction)
+        return np.array(predictions)
 
     def save(self, name: str):
         """Saves the model
